#!/bin/bash

#Company: Trenz Electronic GmbH
#Engineer: Mohsen Chamanbaz

#check tools
TOOLS=( \
    dd                  \
    losetup             \
    parted              \
    lsblk               \
    mkfs.vfat           \
    mkfs.ext4           \
    debootstrap         \
    gzip                \
    cpio                \
    sync                \
    chroot              \
    apt-get             \
    dpkg-reconfigure    \
    sed                 \
    locale-gen          \
    update-locale       \
    qemu-arm-static     \
)
    
STOP=0    
for (( i = 0; i < ${#TOOLS[@]}; i++ )); do
    TOOL=`which ${TOOLS[$i]}`
    if [ -z ${TOOL} ]; then
        echo -e "${TOOLS[$i]} ... \e[91mMISSING\e[0m"
        STOP=1
    else
        echo -e "${TOOLS[$i]} ... \e[92mINSTALLED\e[0m"    
    fi
done

if [ $STOP -eq 1 ]; then 
    echo "stop"
    exit 
fi



petadir=.
image=te0808-ubuntu.img
size=14000




# create an empty image
dd if=/dev/zero of=$image bs=1M count=$size status=progress

#get a free loop device
device=`losetup -f`
echo "++ Setting up device: $device"
losetup $device $image

ubuntu_version=18.04.2
root_tar=ubuntu-base-${ubuntu_version}-base-arm64.tar.gz
root_url=http://cdimage.ubuntu.com/ubuntu-base/releases/${ubuntu_version}/release/$root_tar


boot_dir=/tmp/BOOT
root_dir=/tmp/ROOT

##########################################################################
# Create partitions

parted -s $device mklabel msdos
parted -s $device mkpart primary fat32 4MB 256MB
parted -s $device mkpart primary ext4 256MB 100%

# get device names from the sorted list (-x NAME) !!!
boot_dev=/dev/`lsblk -lno NAME -x NAME $device | sed '2!d'`
root_dev=/dev/`lsblk -lno NAME -x NAME $device | sed '3!d'`


# Create file systems

mkfs.vfat -v $boot_dev
mkfs.ext4 -F -j $root_dev

# Mount file system

mkdir -p $boot_dir $root_dir

mount $boot_dev $boot_dir
mount $root_dev $root_dir



# Copy files to the boot file system

#cp $petadir/images/linux/image.ub $boot_dir
cp $petadir/images/linux/Image $boot_dir
cp $petadir/images/linux/system.dtb $boot_dir


#Generating Device Tree blob file for  rootfile system on Harddisk (system_harddisk.dtb)
dtc -I dtb -O dts -o $boot_dir/system_sd.dts $boot_dir/system.dtb
dtc -I dts -O dtb -o $boot_dir/system_sd.dtb $boot_dir/system_sd.dts	
sed -i s/mmcblk1p2/sda/g $boot_dir/system_sd.dts
dtc -I dts -O dtb -o $boot_dir/system_harddisk.dtb $boot_dir/system_sd.dts
rm -r $boot_dir/system_sd.dts
rm -r $boot_dir/system.dtb

################################################################################
echo "Building Ubuntu ${ubuntu_version} rootfs for Zynq Ultrascale+" 

wget -c $root_url
tar xfvp ./$root_tar -C $root_dir
#tar xvfz ./$root_tar -C $root_dir

sync
chown root:root $root_dir
chmod 755 $root_dir


passwd=root
timezone=Europe/Berlin

#################################################################################################
# Add missing configuration files and packages

cp /etc/resolv.conf $root_dir/etc/
cp /usr/bin/qemu-aarch64-static $root_dir/usr/bin/

mkdir -p $root_dir/usr/local/bin
mkdir -p $root_dir/usr/local/sbin

mkdir -p $root_dir/lib/modules

# TE audio for TE0803
#cp -av $petadir/build/tmp/sysroots/plnx_arm/lib/modules/4.19.0-xilinx-v2019.2/ $root_dir/lib/modules

# get Xilinx modules
startdir=`pwd`
cd $petadir/build/tmp/deploy/images/plnx-zynqmp/
# gzip -dk --force petalinux-user-image-plnx-zynqmp.cpio.gz
mkdir tmp
cd tmp
cpio -id < ../petalinux-user-image-plnx-zynqmp.cpio
cp -av lib/modules/4.19.0-xilinx-v2019.2/ $root_dir/lib/modules

sync
cd ..
rm -rf tmp
# rm -f petalinux-user-image-plnx-zynqmp.cpio
cd $startdir


modules_path=`find build/tmp/work/plnx_zynqmp-xilinx-linux/linux-xlnx/ -maxdepth 5 -type d -print | grep image/lib/modules/4.19.0-xilinx-v2019.2`
cp -av $petadir/$modules_path/ $root_dir/lib/modules

X11DIR=$root_dir/usr/share/X11/xorg.conf.d
mkdir -p $X11DIR
cat << EOF_FBDEV > $X11DIR/99-fbdev.conf
Section "Device"  
  Identifier "myfb"
  Driver "fbdev"
  Option "fbdev" "/dev/fb0"
EndSection
EOF_FBDEV

#echo te-audio-codec >> $root_dir/etc/modules
#################################################################################################


chroot $root_dir <<- EOF_CHROOT


  	export LANG=C
	export LC_ALL=C

		
	
	cat <<- EOF_CAT > etc/apt/apt.conf.d/99norecommends
	APT::Install-Recommends "0";
	APT::Install-Suggests "0";
	EOF_CAT

		
	mkdir /media/BOOT
	mkdir /media/ROOTFS

	cat <<- EOF_CAT > etc/fstab
	# /etc/fstab: static file system information.
	# <file system> <mount point>               <type>    <options>      <dump> <pass>
	#/dev/mmcblk1p2 /dev/disk/by-label/ROOTFS    ext4  errors=remount-ro    0       1		
	#/dev/mmcblk1p1 /dev/disk/by-label/BOOT      vfat     defaults          0       2
	/dev/mmcblk1p2 /media/ROOTFS                 ext4  errors=remount-ro    0       1		
	/dev/mmcblk1p1 /media/BOOT                   vfat     defaults          0       2		
	EOF_CAT

	
	
	cat <<- EOF_CAT >> etc/securetty
	# Serial Console(s) for Xilinx Zynq Ultrascale+
	ttyPS0
	ttyPS1
	EOF_CAT

	echo zynqmp > etc/hostname

	apt-get update
	apt-get -y upgrade



	apt-get -y install locales

	sed -i "/^# en_US.UTF-8 UTF-8$/s/^# //" etc/locale.gen
	locale-gen
	update-locale LANG=en_US.UTF-8


	echo $timezone > etc/timezone
	dpkg-reconfigure --frontend=noninteractive tzdata

	#------------------------ Essential------------------------------------------------------
	apt-get -y install nano mplayer openssh-server  ntp ntpdate  \
	  usbutils lsof parted curl wget vim  ifplugd \
	  python3 python3-numpy build-essential libfftw3-dev python3-scipy  \
	  tcpdump ethtool net-tools joe mc usbutils nodejs mousepad scilab-cli \
	  net-tools iptables iputils-ping man dhcpcd5 konsole firefox
	#-----------------------------------------------------------------------------------------

	#---------------------------Ubuntu-----------------------------------------------------------
	apt-get -y install   nm-tray xorg xauth xinit xserver-xorg-video-fbdev \
		kubuntu-desktop language-pack-kde-de lightdm 
	#--------------------------------------------------------------------------------------	
	#dpkg-reconfigure lxdm
	#-------------------------- Debian-----------------------------------------------------------
	#apt-get -y install   xfonts-base tightvncserver xfce4-panel xfce4-session xfwm4 xfdesktop4 \
	#	 xfce4-terminal xserver-xorg xinit xserver-xorg-video-fbdev
	#--------------------------------------------------------------------------------------------
	

	# enable root login via ssh
	sed -i 's/^#PermitRootLogin.*/&\nPermitRootLogin yes/' etc/ssh/sshd_config
	touch etc/udev/rules.d/75-persistent-net-generator.rules


	#------------------------------Activating DHCP --------------------------------------------

	touch /etc/dhcpcd.conf
	
	cat <<- EOF_CAT > /etc/dhcpcd.conf
	
	# A sample configuration for dhcpcd.
	# See dhcpcd.conf(5) for details.

	# Allow users of this group to interact with dhcpcd via the control socket.
	#controlgroup wheel

	# Inform the DHCP server of our hostname for DDNS.
	hostname

	# Use the hardware address of the interface for the Client ID.
	#clientid
	# or
	# Use the same DUID + IAID as set in DHCPv6 for DHCPv4 ClientID as per RFC4361.
	# Some non-RFC compliant DHCP servers do not reply with this set.
	# In this case, comment out duid and enable clientid above.
	duid

	# Persist interface configuration when dhcpcd exits.
	persistent


	# Rapid commit support.
	# Safe to enable by default because it requires the equivalent option set
	# on the server to actually work.
	option rapid_commit

	# A list of options to request from the DHCP server.
	option domain_name_servers, domain_name, domain_search, host_name
	option classless_static_routes
	# Most distributions have NTP support.
	option ntp_servers


	# Respect the network MTU. This is applied to DHCP routes.
	option interface_mtu

	# A ServerID is required by RFC2131.
	require dhcp_server_identifier

	# Generate Stable Private IPv6 Addresses instead of hardware based ones
	slaac private

	#denyinterfaces eth0


	auto lo
	iface lo inet loopback


	interface eth0
	auto eth0
	allow-hotplug eth0
	iface eth0 inet dhcp

	EOF_CAT
	
	#------------------------------------------------------------------------

	service dhcpcd start
	systemctl enable dhcpcd
 


	#------------------------ Automatic Executing Startx---------------------
	touch /root/
	cat <<- EOF_CAT > /root/.profile

	# ~/.profile: executed by Bourne-compatible login shells.

	if [ "$BASH" ]; then
	  if [ -f ~/.bashrc ]; then
	    . ~/.bashrc
	  fi
	fi

	tty -s && mesg n || true
	
	#exec startx
	

	EOF_CAT
	#-------------------------------------------------------------------------

	cat <<- EOF_CAT > etc/default/ifplugd
	INTERFACES="eth0"
	HOTPLUG_INTERFACES=""
	ARGS="-q -f -u0 -d10 -w -I"
	SUSPEND_ACTION="stop"
	EOF_CAT


	apt-get clean
	echo root:$passwd | chpasswd
	service ntp stop
	history -c

EOF_CHROOT


rm $root_dir/etc/resolv.conf
rm $root_dir/usr/bin/qemu-aarch64-static

sync

# Unmount file systems
umount -f $boot_dir $root_dir

sleep 1

# rmdir $boot_dir $root_dir
rm -rf $boot_dir $root_dir

losetup -d $device

sync




